
/**
 * Copyright 2012 Clint Combs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.newhart.example

import org.newhart._
import org.newhart.mongodb.AuditStore
import java.util.{Date => JDate}
import com.mongodb.casbah.Imports._

object AuditAPIExample1 extends App with AuditAPI {
  implicit val con = MongoConnection("localhost")
  implicit val store = new AuditStore()

  withAudit("users") {
    auditOrigin("origin")
    auditOriginKey("originKey")
    auditOriginKeyType("originKeyType")
    auditOriginHostname("locahost")
    auditCriticality(Criticality.major)
    auditTS(new JDate())
    auditCreateTS(new JDate())
    auditUpdateTS(new JDate())
    auditMsg("message1")
    auditError("an error occurred")
    auditError(new Exception("oops"))
    auditWarning("warning!")
    auditLabel("usersLabel1")
    auditData("test" -> DBObject("a" -> 1))
    auditData(DBObject("b" -> 2, "c" -> 3))
    auditData("users", "testAgain" -> DBObject("a" -> 1))
    auditData("users", DBObject("bAgain" ->2, "cAgain" -> 3))
  }
}

object AuditAPIExample2 extends App with AuditAPI {
  implicit val con = MongoConnection("localhost")
  implicit val store = new AuditStore()

  withAudit("test") {
    auditOrigin("origin")
    auditOriginKey("originKey")
    auditOriginKeyType("originKeyType")
    //auditOriginHostname("locahost")
    auditCriticality(Criticality.major)
    //auditTS(new JDate())
    //auditCreateTS(new JDate())
    auditUpdateTS(new JDate())
    auditMsg("message1")
    /*
    auditError("an error occurred")
    auditError(new Exception("oops"))
    auditWarning("warning!")
    auditLabel("usersLabel1")
    */
  }
}

object AuditAPIExample3 extends App with AuditAPI {
  implicit val con = MongoConnection("localhost")
  implicit val store = new AuditStore()

  case class Sample(value:String)

  withAudit("users") {
    val s = Sample("testValueForUsers")
    auditOrigin("AuditAPIExample3")
    auditOriginKeyType(classOf[Sample].toString)
    auditOriginKey(s.toString)

    auditError("an error occurred")
    auditError(new Exception("oops"))
    auditWarning("warning!")
    auditLabel("usersLabel1")

    withAudit("coders") {
      val s = Sample("testValueForCoders")
      auditOrigin("AuditAPIExample3")
      auditOriginKeyType(classOf[Sample])
      auditOriginKey(s.toString)

      withAudit("a") {
        val s = Sample("testValueForA")
        auditOrigin("AuditAPIExample3")
        auditOriginKeyType(classOf[Sample].toString)
        auditOriginKey(s.toString)

        // TODO: writing an error for coders will require an auditError method that takes an auditName parameter
        auditError("an error for coders")
      }

      withAudit("managers") {
        val s = Sample("testValueForManagers")
        auditOrigin("AuditAPIExample3")
        auditOriginKeyType(classOf[Sample].toString)
        auditOriginKey(s.toString)

        auditWarning("a warning for managers")
      }
    }

    throw new Exception("thrown inside users block, after coders block")
  }
}

object AuditAPIExample4 extends App with AuditAPI {
  implicit val con = MongoConnection("localhost")
  implicit val store = new AuditStore()

  fun0()

  def fun0() = {
    withAudit("test") {
      auditOrigin("origin")
      auditOriginKey("originKey")
      auditOriginKeyType("originKeyType")
      auditCriticality(Criticality.major)
      auditUpdateTS(new JDate())
      auditMsg("message from fun0")
      fun1()
    }
  }

  def fun1() = {
    withAudit("a") {
      auditOrigin("originA")
      auditOriginKey("originKeyA")
      auditOriginKeyType("originKeyTypeA")
      auditCriticality(Criticality.major)
      auditUpdateTS(new JDate())
      auditMsg("message from fun1 in 'a'")

      withAudit("test") {
        auditError("an error from fun1")
        AuditAPIExample4a.fun2()

        auditMsg("a", "message from within 'test' block about 'a'")
      }
    }
  }
}

object AuditAPIExample4a extends AuditAPI {
  def fun2()(implicit store:AuditStore) = {
    auditError("an error from AuditAPIExample4a.fun2() at top level of fun2() function")

    withNewAudit("test") {
      auditOrigin("origin here")
      auditOriginKey("originKey here")
      auditOriginKeyType("originKeyType here")
      auditCriticality(Criticality.major)
      auditUpdateTS(new JDate())
      auditMsg("message from fun2")
      auditError("another error from AuditAPIExample4a.fun2() inside 'test' block")
      auditWarning("warning message from AuditAPIExample4a.fun2()")

      auditMsg("a", "message from within AuditAPIExample4a.fun2() 'test' block about 'a'")
    }
  }
}
