
/**
 * Copyright 2012 Clint Combs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.newhart.example

import org.newhart._
import org.newhart.mongodb.AuditStore
import java.util.{Date => JDate}
import com.mongodb.casbah.Imports._
import scopt.immutable.OptionParser
import com.weiglewilczek.slf4s.Logging

object SearchExample extends App with Logging {

  val parser = new OptionParser[SearchExampleConfig]("SearchExample", "0.0.1-SNAPSHOT") {
    def options = Seq(
      arg("<hostname>", "the MongoDB hostname") {
        (s:String, cfg:SearchExampleConfig) => cfg.copy(hostname = s)
      },
      arg("<collection>", "the collection name") {
        (s:String, cfg:SearchExampleConfig) => cfg.copy(collection = s)
      },
      intOpt("l", "limit", "limit the number of results") {
        (i: Int, cfg: SearchExampleConfig) => cfg.copy(limit = if (i > 0) Some(i) else None)
      }
    )
  }

  parser.parse(args, SearchExampleConfig()) map { cfg =>
    logger.info(cfg.toString)
    run(cfg)
  } getOrElse {
    // arguments are bad, usage message will have been displayed
  }

  def run(cfg:SearchExampleConfig) = {
    implicit val con = MongoConnection(cfg.hostname)
    implicit val store = new AuditStore()

    val q = MongoDBObject()
    println("search")
    store.search(cfg.collection, q, MongoDBObject(), cfg.limit) foreach {
      println
    }

    println("searchIdEntryPairs")
    store.searchIdEntryPairs(cfg.collection, q, MongoDBObject(), cfg.limit) foreach {
      println
    }

    con.close
  }
}

case class SearchExampleConfig(
  hostname:String = "localhost", collection:String = "example", limit:Option[Int] = None)
